unit header_utils;

// Header file utilities

{$mode objfpc}{$H+}
interface

uses
  Classes, SysUtils, StrUtils, Forms, Graphics, dateutils,
  appsettings, Unit4, Unit5,
  synaser, synautil, blcksock, tlntsend,
  PrintersDlgs, Printers, Process;

function OpenComm() : boolean;
function CloseComm() : boolean;
procedure WriteDLHeader(Style:String; Setting:String='');
function SendGet(command:string; LeaveOpen:boolean = False; Timeout:Integer=3000; GetAlso:boolean = True; HideStatus:boolean = False) : string;
function GetReading(): String;
procedure StatusMessage(Mstring:string; BackColor:TColor = clBtnFace);
procedure GetVersion;
procedure CheckLockVisibility;
procedure UpdateCalReport;
procedure PrintLine(LabelText:String; DataText: String='');


implementation

uses
  Unit1,
  vinfo;


// Open the communications port
//Will only open if not already opened
function OpenComm() : boolean;
begin
     //Check selected comm. tab
     if SelectedInterface ='USB' then               //USB
       begin
              ser.LinuxLock:=False; //lock file sometimes persists stuck if program closes before port
              SelectedPort:=Form1.USBPort.Text;
              ser.Connect(SelectedPort);
              //sleep(1000);
              ser.config(115200, 8, 'N', SB1, False, False);
              PortName:=SelectedPort;
       end;
     if SelectedInterface ='Eth' then               //Ethernet
       begin
              EthSocket := TTCPBlockSocket.Create;
              EthSocket.ConvertLineEnd := True;
              //Try setting the timeout a bit longer than normal, because
              // finding Ethernet devices failed a few times on Windows7 netbook.
              EthSocket.SetRecvTimeout(2000);
              EthSocket.SetSendTimeout(2000);
              SelectedIP:=Form1.EthernetIP.Text;
              SelectedPort:=Form1.EthernetPort.Text;
              EthSocket.Connect(SelectedIP, SelectedPort);
              PortName:=SelectedIP;
          end;
     if SelectedInterface ='RS232' then               //RS232
        begin
              ser.Connect(Form1.RS232Port.Text);
              ser.config(StrToIntDef(Form1.RS232Baud.Text,115200), 8, 'N', SB1, False, False);
              PortName:=Form1.RS232Port.Text;
        end;
     OpenComm:=True;    //Indicate success
end;
// Close the communications port
function CloseComm() : boolean;
begin
     //Check selected comm. tab
     if (SelectedInterface ='USB') or (SelectedInterface ='RS232') then  //USB or RS232
       begin
        ser.CloseSocket;
        //ser.Free;
        end;
     if SelectedInterface ='Eth' then               //Ethernet
       begin
            EthSocket.CloseSocket;
            EthSocket.Free;
       end;
     CloseComm:=True;       //Indicate success
end;


procedure WriteDLHeader(Style:String; Setting: String = '' );
var
  result, HeaderFirmwareVersion: AnsiString;
  ProtocolNumber,ModelNumber,FeatureNumber,SerialNumber : Integer;
  Info:     TVersionInfo;

begin

    {Gather information about the selected unit}
    result:=SendGet('ix');
    ProtocolNumber:=StrToIntDef(AnsiMidStr(result,3,8),0);
    ModelNumber:=StrToIntDef(AnsiMidStr(result,12,8),0);
    FeatureNumber:=StrToIntDef(AnsiMidStr(result,21,8),0);
    SerialNumber:=StrToIntDef(AnsiMidStr(result,30,8),0);
    HeaderFirmwareVersion:=
      IntToStr(ProtocolNumber)+'-'+
      IntToStr(ModelNumber)+'-'+
      IntToStr(FeatureNumber);

    LogFileName:=Format('%s%s_%s.dat',[appsettings.LogsDirectory + DirectorySeparator,FormatDateTime('yyyymmdd"_"hhnnss',Now),DLHeaderForm.InstrumentIDEntry.Text]);
    Application.ProcessMessages;
    AssignFile(DLRecFile,LogFileName);

    Rewrite(DLRecFile); //Open file for writing

    { Write header }
    Writeln(DLRecFile,'# Light Pollution Monitoring Data Format 1.0');
    Writeln(DLRecFile,'# URL: http://www.darksky.org/measurements');
    Writeln(DLRecFile,'# Number of header lines: 35');
    Writeln(DLRecFile,'# This data is released under the following license: ODbL 1.0 http://opendatacommons.org/licenses/odbl/summary/');
    Writeln(DLRecFile,'# Device type: '+SelectedModelDescription);
    Writeln(DLRecFile,'# Instrument ID: '+DLHeaderForm.InstrumentIDEntry.Text);
    Writeln(DLRecFile,'# Data supplier: '+DLHeaderForm.DataSupplierEntry.Text);
    Writeln(DLRecFile,'# Location name: '+DLHeaderForm.LocationNameEntry.Text);
    Writeln(DLRecFile,'# Position (lat, lon, elev(m)): '+DLHeaderForm.PositionEntry.Text);
    Writeln(DLRecFile,'# Local timezone: '+DLHeaderForm.TZLocationBox.Text);
    Writeln(DLRecFile,'# Time Synchronization: '+DLHeaderForm.TimeSynchEntry.Text);
    Writeln(DLRecFile,'# Moving / Stationary position: '+DLHeaderForm.MovingStationaryPositionCombo.Text);
    Writeln(DLRecFile,'# Moving / Fixed look direction: '+DLHeaderForm.MovingStationaryDirectionCombo.Text);
    Writeln(DLRecFile,'# Number of channels: '+DLHeaderForm.NumberOfChannelsEntry.Text);
    Writeln(DLRecFile,'# Filters per channel: '+DLHeaderForm.FiltersPerChannelEntry.Text);
    Writeln(DLRecFile,'# Measurement direction per channel: '+DLHeaderForm.MeasurementDirectionPerChannelEntry.Text);
    Writeln(DLRecFile,'# Field of view (degrees): '+DLHeaderForm.FieldOfViewEntry.Text);
    if Style = 'DL-Log' then {Short record}
      Writeln(DLRecFile,'# Number of fields per line: 5')
    else {Long record}
       Writeln(DLRecFile,'# Number of fields per line: 6');
    Writeln(DLRecFile,Format('# SQM serial number: %d',[SerialNumber]));
    Writeln(DLRecFile,'# SQM firmware version: '+HeaderFirmwareVersion);
    Writeln(DLRecFile,'# SQM cover offset value: '+DLHeaderForm.CoverOffsetEntry.Text);
    Writeln(DLRecFile,'# SQM readout test ix: '+result);
    Writeln(DLRecFile,'# SQM readout test rx: '+sendget('rx'));
    Writeln(DLRecFile,'# SQM readout test cx: '+sendget('cx'));
    Writeln(DLRecFile,'# Comment: '+DLHeaderForm.UserComment1.Text);
    Writeln(DLRecFile,'# Comment: '+DLHeaderForm.UserComment2.Text);
    Writeln(DLRecFile,'# Comment: '+DLHeaderForm.UserComment3.Text);
    Writeln(DLRecFile,'# Comment: '+DLHeaderForm.UserComment4.Text);
    Writeln(DLRecFile,'# Comment: '+DLHeaderForm.UserComment5.Text);

    // Log the UDM version.
    Info := TVersionInfo.Create;
    Info.Load(HINSTANCE);
    Writeln(DLRecFile,Format('# UDM version: %s',
                                [IntToStr(Info.FixedInfo.FileVersion[0])
                                +'.'+IntToStr(Info.FixedInfo.FileVersion[1])
                                +'.'+IntToStr(Info.FixedInfo.FileVersion[2])
                                +'.'+IntToStr(Info.FixedInfo.FileVersion[3])]));
    Info.Free;

    //Log the current UDM settings that were passed here.
    Writeln(DLRecFile,Format('# UDM setting: %s',[Setting]));

    Writeln(DLRecFile,'# blank line 32');
    if Style = 'DL-Log' then {Short record}
      begin
        Writeln(DLRecFile,'# UTC Date & Time, Local Date & Time, Temperature, Voltage, MSAS');
        Writeln(DLRecFile,'# YYYY-MM-DDTHH:mm:ss.fff;YYYY-MM-DDTHH:mm:ss.fff;Celsius;Volts;mag/arcsec^2');
      end
    else {Long record}
      begin
          Writeln(DLRecFile,'# UTC Date & Time, Local Date & Time, Temperature, Counts, Frequency, MSAS');
          Writeln(DLRecFile,'# YYYY-MM-DDTHH:mm:ss.fff;YYYY-MM-DDTHH:mm:ss.fff;Celsius;number;Hz;mag/arcsec^2');
      end;
    Writeln(DLRecFile,'# END OF HEADER');
    Flush(DLRecFile);
end;

// Send a command strings then return the result
function SendGet(command:string; LeaveOpen:boolean = False; Timeout:Integer=3000; GetAlso:boolean = True; HideStatus:boolean = False) : string;
//LeaveOpen indicates that the communication port should be left open
var
   ErrorString: AnsiString;
begin

     //Initialze output string to nothing.
     SendGet:='';

     if (not LeaveOpen) then
        begin
                OpenComm;
        end;
     ErrorString:='';
     //Check selected comm. tab
     if (SelectedInterface ='USB') or (SelectedInterface ='RS232') then //USB or RS232
       begin
          //if ser.CanWrite(10) then
             //Can write
          //else
             //Can not write
          //ser.InstanceActive
          ser.SendString(command);
          if (GetAlso) then
             SendGet:=ser.Recvstring(Timeout);
          If CompareStr(ser.LastErrorDesc,'OK')<>0 then
             ErrorString:='Error: '+ser.LastErrorDesc;
       end;
     if SelectedInterface ='Eth' then               //Ethernet
        begin
          EthSocket.SendString(command);
          if (GetAlso) then
             SendGet:=EthSocket.RecvString(1000);
          If CompareStr(EthSocket.LastErrorDesc,'OK')<>0 then
             ErrorString:=EthSocket.LastErrorDesc;
        end;

     if (not LeaveOpen) then
        begin
                CloseComm;
        end;
     if not HideStatus then
        begin
          if GetAlso then
            StatusMessage('Sent: '+command+'   To: '+PortName+'   Received: '+SendGet+ErrorString)
          else
            StatusMessage('Sent: '+command+'   To: '+PortName);
        end;
end;

function GetReading(): String;
var
   //result:string;
   pieces: TStringList;
begin
  //Clear out existing results
  Form1.ReadingListBox.Items.Clear;

  //Try to ensure a model version has been found.
  if SelectedModel=0 then
     GetVersion;

  //Get response to "Request"
  pieces := TStringList.Create;
  pieces.Delimiter := ',';
  result:=SendGet('rx');
  pieces.DelimitedText := result;

  case SelectedModel of
    3,5,6,7: begin   //3=Standard SQM-LE/LU, 5=SQM-LR, 6=SQM-LU-DL, 7 =SQM-LU-GPS
       if pieces.count=6 then
         begin
           Form1.ReadingListBox.Items.Add(Format('  Reading: %1.2fmpsas',[StrToFloatDef(AnsiMidStr(pieces.Strings[1],1,5),0,FPointSeparator)]));
           Form1.ReadingListBox.Items.Add(Format('Frequency: %dHz',      [StrToIntDef  (AnsiMidStr(pieces.Strings[2],1,10),0)]));
           Form1.ReadingListBox.Items.Add(Format('  Counter: %dcounts',  [StrToIntDef  (AnsiMidStr(pieces.Strings[3],1,10),0)]));
           Form1.ReadingListBox.Items.Add(Format('     Time: %1.3fs',    [StrToFloatDef(AnsiMidStr(pieces.Strings[4],1,11),0,FPointSeparator)]));
           Form1.ReadingListBox.Items.Add(Format('     Tint: %1.1fC',    [StrToFloatDef(AnsiMidStr(pieces.Strings[5],1,5),0,FPointSeparator)]));
         end
       else
           Form1.ReadingListBox.Items.Add('No response');
    end;
    8: begin         //8=Magnetometer
       if pieces.count=3 then
       begin
         Form1.ReadingListBox.Items.Add(Format('M1: %dc',    [StrToIntDef(AnsiMidStr(pieces.Strings[1],1,10),0)]));
         if StrToIntDef(pieces.Strings[2],0) < 4096 then
            Form1.ReadingListBox.Items.Add(Format('T1: %10.7fC',[StrToFloatDef(pieces.Strings[2],0,FPointSeparator)/128.0]))
         else
             Form1.ReadingListBox.Items.Add(Format('T1: %10.7fC',[(StrToFloatDef(pieces.Strings[2],0,FPointSeparator)-65536.0)/128.0]));

       end
       else
         Form1.ReadingListBox.Items.Add('No response');
    end;
    10: begin         //10=Temperature chamber
       if pieces.count=3 then
       begin
         Form1.ReadingListBox.Items.Add(Format('M1: %dc',    [StrToIntDef(AnsiMidStr(pieces.Strings[1],1,10),0)]));
         if StrToIntDef(pieces.Strings[2],0) < 4096 then
            Form1.ReadingListBox.Items.Add(Format('T1: %10.7fC',[StrToFloatDef(pieces.Strings[2],0,FPointSeparator)/128.0]))
         else
             Form1.ReadingListBox.Items.Add(Format('T1: %10.7fC',[(StrToFloatDef(pieces.Strings[2],0,FPointSeparator)-65536.0)/128.0]));

       end
       else
         Form1.ReadingListBox.Items.Add('No response');
    end;
    1: begin         //1=ADA
       if pieces.count=8 then
         begin
           Form1.ReadingListBox.Items.Add(Format('Frequency: %dHz',      [StrToIntDef(AnsiMidStr(pieces.Strings[1],1,10),0)]));
           Form1.ReadingListBox.Items.Add(Format(' Counter1: %dcounts',  [StrToIntDef(AnsiMidStr(pieces.Strings[2],1,10),0)]));
           Form1.ReadingListBox.Items.Add(Format('    Time1: %1.3fs',    [StrToFloatDef(AnsiMidStr(pieces.Strings[3],1,11),0,FPointSeparator)]));
           Form1.ReadingListBox.Items.Add(Format(' Counter2: %dcounts',  [StrToIntDef(AnsiMidStr(pieces.Strings[4],1,10),0)]));
           Form1.ReadingListBox.Items.Add(Format('    Time2: %1.3fs',    [StrToFloatDef(AnsiMidStr(pieces.Strings[5],1,11),0,FPointSeparator)]));
         end
       else
           Form1.ReadingListBox.Items.Add('No response');
    end;
    4: begin         //4=Colour

    end;
  otherwise
     Form1.ReadingListBox.Items.Add('Could not get version.');
  end;

end;

procedure GetVersion;
{$IFDEF Linux}
const
     READ_BYTES = 2048;
{$ENDIF}
var
   result:string;
   TempOut:String;
   pieces: TStringList;
   ResultCount:Integer;
   MyBuffer : array[0..99999] of char;
   OurProcess: TProcess;
   MemStream: TMemoryStream;
   OutputLines: TStringList;
   NumBytes: LongInt;
   BytesRead: LongInt;
begin

  if not(gettingversion) then
   begin

     gettingversion:=True;
     //Clear out existing results
     Form1.VersionListBox.Items.Clear;

     ResultCount:=0;

     pieces := TStringList.Create;
     pieces.Delimiter := ',';

     result:=SendGet('ix');
     pieces.DelimitedText := result;

     //Check size of array. 5 Sections normally.
     //There is a case where other software might be accessing the device,
     //  another immediate request should get the proper data, and a warning
     //  that possibly other software is accessing the device.
     if ((pieces.Count>=5) and (pieces.Strings[0]='i')) then
       begin
            ResultCount:=1;
       end
     else //Try once again. Sometimes dual responses get through here.
         begin
         result:=SendGet('ix');
         pieces.DelimitedText := result;
          if ((pieces.Count>=5) and (pieces.Strings[0]='i')) then
               ResultCount:=2
          else
             begin
             { Check failed dialout usb connection; groups, suggest adduser then relogin. or administration->users and groups->manage groups->dialout->properties}
               {$ifdef Linux}

                 MemStream := TMemoryStream.Create;
                 BytesRead := 0;

                 pieces := TStringList.Create;
                 pieces.Delimiter := '=';
                 OurProcess := TProcess.Create(nil);
                 OurProcess.Executable := 'groups';
                 OurProcess.Options := [poUsePipes];
                 OurProcess.Execute;
                 while OurProcess.Running do
                 begin
                   MemStream.SetSize(BytesRead + READ_BYTES);
                   NumBytes := OurProcess.Output.Read((MemStream.Memory + BytesRead)^, READ_BYTES);
                   if NumBytes > 0
                   then begin
                     Inc(BytesRead, NumBytes);
                   end
                   else begin
                     Sleep(100);
                   end;
                 end;
                 repeat
                   MemStream.SetSize(BytesRead + READ_BYTES);
                   NumBytes := OurProcess.Output.Read((MemStream.Memory + BytesRead)^, READ_BYTES);
                   if NumBytes > 0
                   then begin
                     Inc(BytesRead, NumBytes);
                   end;
                 until NumBytes <= 0;
                   if BytesRead > 0 then
                      WriteLn;
                   MemStream.SetSize(BytesRead);

                   OutputLines := TStringList.Create;
                   OutputLines.LoadFromStream(MemStream);

                   if not AnsiContainsStr(OutputLines[0],'dialout') then
                     begin
                       Form1.VersionListBox.Items.Add('This user is not part of the');
                       Form1.VersionListBox.Items.Add('  dialout group.');
                       Form1.VersionListBox.Items.Add('Use the "adduser" command, or:');
                       Form1.VersionListBox.Items.Add('  administration->');
                       Form1.VersionListBox.Items.Add('    users and groups->');
                       Form1.VersionListBox.Items.Add('      manage groups->');
                       Form1.VersionListBox.Items.Add('        dialout->');
                       Form1.VersionListBox.Items.Add('          properties');
                       Form1.VersionListBox.Items.Add('add user name, then re-login');
                       StatusMessage('User not in dialout group.');
                     end;

                   OutputLines.Free;
                   OurProcess.Free;
                   MemStream.Free;
               {$else}
                 Form1.VersionListBox.Items.Add('Other software may be ');
                 Form1.VersionListBox.Items.Add('accessing the device!');
                 StatusMessage('Other software may be accessing the device!');
               {$endif}
             end;
         end;

     if ResultCount>0 then
       begin
         SelectedProtocol:=IntToStr(StrToIntDef(pieces.Strings[1],0));
         Form1.VersionListBox.Items.Add('Protocol: '+ SelectedProtocol);
         Case StrToIntDef(pieces.Strings[2],0) of
                 1: SelectedModelDescription:='ADA';
                 3: begin
                    if Form1.CommNotebook.PageIndex=0 then
                       SelectedModelDescription:='SQM-LU'
                    else
                       SelectedModelDescription:='SQM-LE';
                 end;
                 4: SelectedModelDescription:='Colour';
                 5: SelectedModelDescription:='SQM-LR';
                 6: SelectedModelDescription:='SQM-LU-DL';
                 7: SelectedModelDescription:='SQM-LU-GPS';
                 8: SelectedModelDescription:='Magnetometer';
                 10: SelectedModelDescription:='Temp. Chamber';
         otherwise
                  SelectedModelDescription:='Unknown';
         end;
         Form1.VersionListBox.Items.Add('   Model: '+ IntToStr(StrToIntDef(pieces.Strings[2],0))+ ' ('+ SelectedModelDescription + ')');
         SelectedFeature:=IntToStr(StrToIntDef(pieces.Strings[3],0));
         Form1.VersionListBox.Items.Add(' Feature: '+ SelectedFeature);
         SelectedUnitSerialNumber:=IntToStr(StrToIntDef(pieces.Strings[4],0));
         DLHeaderForm.SerialNumber.Text:=SelectedUnitSerialNumber;
         DLHeaderForm.ReadINI;
         Form1.VersionListBox.Items.Add('  Serial: '+ SelectedUnitSerialNumber);

         SelectedModel:=StrToIntDef(pieces.Strings[2],0);

       //Datalogging tab: 6=SQM-LU-DL, 7 =SQM-LU-GPS
       if ((SelectedModel=6)or (SelectedModel=7)) then
       begin
            Form1.DataNoteBook.Page[4].TabVisible:=True;
            DataLoggingAvailable:=True;
       end
       else
       begin
            Form1.DataNoteBook.Page[4].TabVisible:=False;
	    DataLoggingAvailable:=False;
       end;

       //GPS tab: 7 =SQM-LU-GPS
       if (SelectedModel=7) then
            Form1.DataNoteBook.Page[6].TabVisible:=True
       else
            Form1.DataNoteBook.Page[6].TabVisible:=False;

       CheckLockVisibility();
     end;

     gettingversion:=False;
   end;
 Application.ProcessMessages;
 UpdateCalReport;


end;

procedure GetVersionOld;
var
   result:string;
   TempOut:String;
   pieces: TStringList;
   ResultCount:Integer;
   MyBuffer : array[0..99999] of char;
begin

  if not(gettingversion) then
   begin

     gettingversion:=True;
     //Clear out existing results
     Form1.VersionListBox.Items.Clear;

     ResultCount:=0;

     pieces := TStringList.Create;
     pieces.Delimiter := ',';

     result:=SendGet('ix');
     pieces.DelimitedText := result;

     //Check size of array. 5 Sections normally.
     //There is a case where other software might be accessing the device,
     //  another immediate request should get the proper data, and a warning
     //  that possibly other software is accessing the device.
     if ((pieces.Count>=5) and (pieces.Strings[0]='i')) then
       begin
            ResultCount:=1;
       end
     else //Try once again. Sometimes dual responses get through here.
         begin
         result:=SendGet('ix');
         pieces.DelimitedText := result;
          if ((pieces.Count>=5) and (pieces.Strings[0]='i')) then
               ResultCount:=2
          else
             begin
             { Check failed dialout usb connection; groups, suggest adduser then relogin. or administration->users and groups->manage groups->dialout->properties}
               {$ifdef Linux}
               Form1.Process1.CommandLine:= 'groups';
               Form1.Process1.Execute;
               while Form1.Process1.Running do; //Wait until command finishes
               writeln('debughere8');
               writeln(Form1.Process1.Output.NumBytesAvailable);
               writeln('debughere9');
               Form1.Process1.Output.Read(MyBuffer[0], Form1.Process1.Output.NumBytesAvailable);
               TempOut:=string(MyBuffer);
               if not AnsiContainsStr('dialout',TempOut) then
                 begin
                   Form1.VersionListBox.Items.Add('This user is not part of the');
                   Form1.VersionListBox.Items.Add('  dialout group.');
                   Form1.VersionListBox.Items.Add('Use the "adduser" command, or:');
                   Form1.VersionListBox.Items.Add('  administration->');
                   Form1.VersionListBox.Items.Add('    users and groups->');
                   Form1.VersionListBox.Items.Add('      manage groups->');
                   Form1.VersionListBox.Items.Add('        dialout->');
                   Form1.VersionListBox.Items.Add('          properties');
                   Form1.VersionListBox.Items.Add('add user name, then re-login');
                 end;
               StatusMessage('User not in dialout group.');
               {$else}
                 Form1.VersionListBox.Items.Add('Other software may be ');
                 Form1.VersionListBox.Items.Add('accessing the device!');
                 StatusMessage('Other software may be accessing the device!');
               {$endif}
             end;
         end;

     if ResultCount>0 then
       begin
         SelectedProtocol:=IntToStr(StrToIntDef(pieces.Strings[1],0));
         Form1.VersionListBox.Items.Add('Protocol: '+ SelectedProtocol);
         Case StrToIntDef(pieces.Strings[2],0) of
                 1: SelectedModelDescription:='ADA';
                 3: begin
                    if Form1.CommNotebook.PageIndex=0 then
                       SelectedModelDescription:='SQM-LU'
                    else
                       SelectedModelDescription:='SQM-LE';
                 end;
                 4: SelectedModelDescription:='Colour';
                 5: SelectedModelDescription:='SQM-LR';
                 6: SelectedModelDescription:='SQM-LU-DL';
                 7: SelectedModelDescription:='SQM-LU-GPS';
                 8: SelectedModelDescription:='Magnetometer';
                 10: SelectedModelDescription:='Temp. Chamber';
         otherwise
                  SelectedModelDescription:='Unknown';
         end;
         Form1.VersionListBox.Items.Add('   Model: '+ IntToStr(StrToIntDef(pieces.Strings[2],0))+ ' ('+ SelectedModelDescription + ')');
         SelectedFeature:=IntToStr(StrToIntDef(pieces.Strings[3],0));
         Form1.VersionListBox.Items.Add(' Feature: '+ SelectedFeature);
         SelectedUnitSerialNumber:=IntToStr(StrToIntDef(pieces.Strings[4],0));
         DLHeaderForm.SerialNumber.Text:=SelectedUnitSerialNumber;
         DLHeaderForm.ReadINI;
         Form1.VersionListBox.Items.Add('  Serial: '+ SelectedUnitSerialNumber);

         SelectedModel:=StrToIntDef(pieces.Strings[2],0);

       //Datalogging tab: 6=SQM-LU-DL, 7 =SQM-LU-GPS
       if ((SelectedModel=6)or (SelectedModel=7)) then
       begin
            Form1.DataNoteBook.Page[4].TabVisible:=True;
            DataLoggingAvailable:=True;
       end
       else
       begin
            Form1.DataNoteBook.Page[4].TabVisible:=False;
	    DataLoggingAvailable:=False;
       end;

       //GPS tab: 7 =SQM-LU-GPS
       if (SelectedModel=7) then
            Form1.DataNoteBook.Page[6].TabVisible:=True
       else
            Form1.DataNoteBook.Page[6].TabVisible:=False;

       CheckLockVisibility();
     end;

     gettingversion:=False;
   end;
 Application.ProcessMessages;
 UpdateCalReport;


end;

procedure StatusMessage(Mstring:string; BackColor:TColor = clBtnFace);

begin
     Form1.StatusBar1.Panels.Items[0].Text:=Mstring;
     Form1.StatusBar1.Color:=BackColor; //Does not seem to work, possibly because of OS theme overriding.
     if not(Mstring='') then
        begin
          ViewedLog.Add(Mstring);
          if ViewingLog then
             Form5.Memo1.Lines.Add(Mstring);
        end;
     { TODO 2 : log file }
     Application.ProcessMessages;
end;

procedure CheckLockVisibility;
begin
  //CheckLock enable/disable
  Form1.CheckLockResult.Text:='';
  if ((SelectedModel=3) and (Form1.CommNotebook.PageIndex=1)) then //3=standard SQM-LE/U, and Ethernet
     begin
       Form1.CheckLock.Visible:=True;
       Form1.CheckLockResult.Visible:=True;
     end
  else
      begin
       Form1.CheckLock.Visible:=False;
       Form1.CheckLockResult.Visible:=False;
      end;

  //Check XPort default button enable/disable
  if (Form1.CommNotebook.PageIndex=1) then //Ethernet
     begin
       Form1.bXPortDefaults.Enabled:=True;
     end
  else
      begin
       Form1.bXPortDefaults.Enabled:=False;
      end;
end;

procedure UpdateCalReport;
begin
     Application.ProcessMessages;
     if (Form1.FoundDevices.SelCount=0) and (Form1.CommNotebook.PageIndex<>2) then //not for RS232
       begin
            Form1.Panel1.Canvas.Clear;
       end
     else
       begin
         PrintingLine:=0;
         PrintLine(SelectedModelDescription+' Calibration data');
         PrintLine('Report Date', FormatDateTime('yyyy-mm-dd',Now));
         PrintLine('Serial Number', SelectedUnitSerialNumber);
         //check if USB device:
         if ((Form1.CommNotebook.PageIndex=0) and not (SelectedModel=5)) then    // USB device but not RS232 model
            PrintLine('USB Serial Number', Form1.USBSerialNumber.text);
         //check if Ethernet device:
         if (Form1.CommNotebook.PageIndex=1) then
            PrintLine('MAC', Form1.EthernetMAC.text);
         PrintLine('Model Number', Inttostr(SelectedModel) + ' ('+SelectedModelDescription+')');
         PrintLine('Feature version', SelectedFeature);
         PrintLine('Protocol version', SelectedProtocol);
         PrintLine('Light calibration offset', Format('%2.2f',[ConfCalmpsas])+' mags/arcsec²');
         PrintLine('Light calibration temperature', Format('%2.1f',[ConfCalLightTemp])+' °C');
         PrintLine('Dark calibration period', Format('%2.3f',[ConfCalPeriod])+' seconds');
         PrintLine('Dark calibration temperature', Format('%2.1f',[ConfCalDarkTemp])+' °C');
         PrintLine('Calibration offset', '8.71 mags/arcsec²');
       end;
end;
procedure PrintLine(LabelText:String; DataText: String='');
var
  H,W,PageTop,PageWidth,CellMargin,CH: Integer;
  WhereTo:TCanvas;
begin
     if CalPrint then //output=printer
       begin
         WhereTo:=Printer.Canvas;
         PageTop:=500;
         CellMargin:=27;
         PageWidth:=Printer.PageWidth;
         H := WhereTo.TextHeight(LabelText);
         W := WhereTo.TextWidth(LabelText);
         CH:=H+2*CellMargin;
         //Draw the rectangle around the text:
         WhereTo.Rectangle(
            Rect(  PageWidth div 5,
                   PageTop + PrintingLine * CH,
                   4 * (PageWidth div 5),
                   PageTop + PrintingLine * CH + CH+1));
       end
     else //output = preview on screen
       begin
            WhereTo:=Form1.Panel1.Canvas;
            PageTop:=1;
            CellMargin:=5;
            PageWidth:=Form1.Panel1.Width;
            H := WhereTo.TextHeight(LabelText);
            W := WhereTo.TextWidth(LabelText);
            CH:=H+2*CellMargin;
            //Draw the rectangle around the text:
            WhereTo.Rectangle(
               Rect(  1,
                      PageTop + PrintingLine * CH,
                      PageWidth-1,
                      PageTop + PrintingLine * CH + CH+1));
       end;

     //Check for two parameters
     if Length(DataText)>0 then
       begin
       //Draw centerline
       WhereTo.Line(
                 PageWidth div 2,
                 PageTop + PrintingLine * CH,
                 PageWidth div 2,
                 PageTop + PrintingLine * CH + CH);
       //Place Left text:
       WhereTo.TextOut((
                 PageWidth div 2) - W - CellMargin,
                 PageTop + (PrintingLine * CH) + CellMargin,
                 LabelText);
       //Place Right text:
       WhereTo.TextOut((
                 PageWidth div 2) + CellMargin,
                 PageTop + (PrintingLine * CH) + CellMargin,
                 DataText);
       end
     else //one text paramater
       begin
       //Place the text:
       WhereTo.TextOut((
                 PageWidth div 2) - (W div 2),
                 PageTop + (PrintingLine * CH) + CellMargin,
                 LabelText);
       end;
     Inc(PrintingLine);
end;


end.

