unit Unit4;

{$mode objfpc}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, Spin, upascaltz,appsettings,
  LCLIntf; //OpenURL
type

  { TDLHeaderForm }

  TDLHeaderForm = class(TForm)
    Button1: TButton;
    CoverOffsetEntry: TLabeledEdit;
    FiltersPerChannelEntry: TComboBox;
    InvalidInstrumentID: TLabel;
    DefinitionsLink: TLabel;
    Label10: TLabel;
    DefinitionsLabel: TLabel;
    SerialNumber: TLabeledEdit;
    UserComment2: TEdit;
    UserComment1: TLabeledEdit;
    MovingStationaryPositionCombo: TComboBox;
    MovingStationaryDirectionCombo: TComboBox;
    Label11: TLabel;
    TZRegionBox: TComboBox;
    TZLocationBox: TComboBox;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    InstrumentIDEntry: TLabeledEdit;
    DataSupplierEntry: TLabeledEdit;
    LocationNameEntry: TLabeledEdit;
    PositionEntry: TLabeledEdit;
    TimeSynchEntry: TLabeledEdit;
    MeasurementDirectionPerChannelEntry: TLabeledEdit;
    FieldOfViewEntry: TLabeledEdit;
    NumberOfChannelsEntry: TSpinEdit;
    UserComment3: TEdit;
    UserComment4: TEdit;
    UserComment5: TEdit;
    procedure Button1Click(Sender: TObject);
    procedure CoverOffsetEntryChange(Sender: TObject);
    procedure DefinitionsLinkClick(Sender: TObject);
    procedure DefinitionsLinkMouseEnter(Sender: TObject);
    procedure DefinitionsLinkMouseLeave(Sender: TObject);
    procedure FiltersPerChannelEntryChange(Sender: TObject);
    procedure FieldOfViewEntryChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MeasurementDirectionPerChannelEntryChange(Sender: TObject);
    procedure MovingStationaryDirectionComboChange(Sender: TObject);
    procedure MovingStationaryPositionComboChange(Sender: TObject);
    procedure DataSupplierEntryChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure InstrumentIDEntryChange(Sender: TObject);
    procedure NumberOfChannelsEntryChange(Sender: TObject);
    procedure TimeSynchEntryChange(Sender: TObject);
    procedure PositionEntryChange(Sender: TObject);
    procedure LocationNameEntryChange(Sender: TObject);
    procedure TZLocationBoxChange(Sender: TObject);
    procedure TZRegionBoxChange(Sender: TObject);
    procedure UserComment1Change(Sender: TObject);
    procedure UserComment2Change(Sender: TObject);
    procedure UserComment3Change(Sender: TObject);
    procedure UserComment4Change(Sender: TObject);
    procedure UserComment5Change(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    procedure ReadINI();
    function CheckInstrumentID() : Boolean;
  end;

var
  INISection: String;
  DLHeaderForm: TDLHeaderForm;
  AZones: TStringList;
  ptz :TPascalTZ;

implementation

uses Unit1;

{ TDLHeaderForm }

procedure TDLHeaderForm.TZLocationBoxChange(Sender: TObject);
begin
     { Save the TZ selection. }
     vConfigurations.WriteString(INISection,'Local time zone',TZLocationBox.Text);
end;

procedure TDLHeaderForm.TZRegionBoxChange(Sender: TObject);
begin
     { Clear out location because region has changed. }
     TZLocationBox.Text:='';

     { Save the TZ region selection. }
     vConfigurations.WriteString(INISection,'Local region',TZRegionBox.Text);

     { Read the region database table. }
     ptz.Destroy;
     ptz := TPascalTZ.Create();
     ptz.ParseDatabaseFromFile(appsettings.TZDirectory+TZRegionBox.Text);
     Azones.Clear;
     ptz.GetTimeZoneNames(AZones,true);
     TZLocationBox.Items.Clear;
     TZLocationBox.Items.AddStrings(AZones);
end;

procedure TDLHeaderForm.FormCreate(Sender: TObject);
begin
     { Initialize required variables. }
     AZones:=TStringList.Create;
     ptz := TPascalTZ.Create();
     CheckInstrumentID;
end;

procedure TDLHeaderForm.DataSupplierEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Data Supplier',DataSupplierEntry.Text);
end;

procedure TDLHeaderForm.MovingStationaryPositionComboChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Moving Stationary Position',MovingStationaryPositionCombo.Text);
end;

procedure TDLHeaderForm.MovingStationaryDirectionComboChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Moving Stationary Direction',MovingStationaryDirectionCombo.Text);
end;

procedure TDLHeaderForm.FiltersPerChannelEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Filters Per Channel',FiltersPerChannelEntry.Text);
end;

procedure TDLHeaderForm.CoverOffsetEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'CoverOffset',CoverOffsetEntry.Text);
end;

procedure TDLHeaderForm.DefinitionsLinkClick(Sender: TObject);
begin
   OpenURL(DefinitionsLink.Caption);
 end;

procedure TDLHeaderForm.DefinitionsLinkMouseEnter(Sender: TObject);
begin
 DefinitionsLink.Cursor := crHandPoint;
 DefinitionsLink.Font.Color := clBlue;
 DefinitionsLink.Font.Style := [fsUnderline];
 if Pos('http://www.', DefinitionsLink.Caption) = 0 then
   DefinitionsLink.Caption := 'http://www.' + DefinitionsLink.Caption;
end;

procedure TDLHeaderForm.DefinitionsLinkMouseLeave(Sender: TObject);
begin
 DefinitionsLink.Font.Style := [];
 if Pos('http://www.', DefinitionsLink.Caption) > 0 then
   DefinitionsLink.Caption := Copy(DefinitionsLink.Caption, Pos('http://www.', DefinitionsLink.Caption) + Length('http://www.'), Length(DefinitionsLink.Caption));

end;

procedure TDLHeaderForm.Button1Click(Sender: TObject);
begin
  Close;
end;

procedure TDLHeaderForm.UserComment1Change(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'UserComment1',UserComment1.Text);
end;

procedure TDLHeaderForm.UserComment2Change(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'UserComment2',UserComment2.Text);
end;

procedure TDLHeaderForm.UserComment3Change(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'UserComment3',UserComment3.Text);
end;

procedure TDLHeaderForm.UserComment4Change(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'UserComment4',UserComment4.Text);
end;

procedure TDLHeaderForm.UserComment5Change(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'UserComment5',UserComment5.Text);
end;

procedure TDLHeaderForm.FieldOfViewEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Field Of View',FieldOfViewEntry.Text);
end;
procedure TDLHeaderForm.ReadINI();
begin
     INISection:='Serial:'+Unit1.SelectedUnitSerialNumber;
     { Pull Timezone information from INI file if it exists.}
     TZRegionBox.Text:= vConfigurations.ReadString(INISection,'Local region');
     if (FileExists(appsettings.TZDirectory+TZRegionBox.Text) and (length(TZRegionBox.Text)>0))then
       begin
         ptz.ParseDatabaseFromFile(appsettings.TZDirectory+TZRegionBox.Text);
         ptz.GetTimeZoneNames(AZones,true);
         TZLocationBox.Items.AddStrings(AZones);
       end;

     { Read the previously recorded entries. }
     TZLocationBox.Text:= vConfigurations.ReadString(INISection,'Local time zone');
     InstrumentIDEntry.Text:=vConfigurations.ReadString(INISection,'Instrument ID');
     DataSupplierEntry.Text:=vConfigurations.ReadString(INISection,'Data Supplier');
     LocationNameEntry.Text:=vConfigurations.ReadString(INISection,'Location Name');
     PositionEntry.Text:=vConfigurations.ReadString(INISection,'Position');
     TimeSynchEntry.Text:=vConfigurations.ReadString(INISection,'Time Synchronization');
     MovingStationaryPositionCombo.Text:=vConfigurations.ReadString(INISection,'Moving Stationary Position');
     MovingStationaryDirectionCombo.Text:=vConfigurations.ReadString(INISection,'Moving Stationary Direction');
     NumberOfChannelsEntry.Text:=vConfigurations.ReadString(INISection,'Number Of Channels');
     FiltersPerChannelEntry.Text:=vConfigurations.ReadString(INISection,'Filters Per Channel');
     MeasurementDirectionPerChannelEntry.Text:=vConfigurations.ReadString(INISection,'Measurement Direction Per Channel');
     FieldOfViewEntry.Text:=vConfigurations.ReadString(INISection,'Field Of View');
     CoverOffsetEntry.Text:=vConfigurations.ReadString(INISection,'CoverOffset');
     UserComment1.Text:=vConfigurations.ReadString(INISection,'UserComment1');
     UserComment2.Text:=vConfigurations.ReadString(INISection,'UserComment2');
     UserComment3.Text:=vConfigurations.ReadString(INISection,'UserComment3');
     UserComment4.Text:=vConfigurations.ReadString(INISection,'UserComment4');
     UserComment5.Text:=vConfigurations.ReadString(INISection,'UserComment5');
end;

procedure TDLHeaderForm.FormShow(Sender: TObject);
begin
end;


procedure TDLHeaderForm.MeasurementDirectionPerChannelEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Measurement Direction Per Channel',MeasurementDirectionPerChannelEntry.Text);
end;

function TDLHeaderForm.CheckInstrumentID() : Boolean;
var
  CheckChar: char;
begin
     CheckInstrumentID:=True; {Assume that text is valid}

     for CheckChar in InstrumentIDEntry.Text do
         if not (CheckChar in ['0'..'9','A'..'Z','a'..'z','_','-']) then
            CheckInstrumentID:=False;

     InvalidInstrumentID.Visible:=not CheckInstrumentID;
end;
procedure TDLHeaderForm.InstrumentIDEntryChange(Sender: TObject);
begin
 Application.ProcessMessages; //Wait for widgets to become visible
 if CheckInstrumentID then
   vConfigurations.WriteString(INISection,'Instrument ID',InstrumentIDEntry.Text);
end;

procedure TDLHeaderForm.NumberOfChannelsEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Number Of Channels',NumberOfChannelsEntry.Text);
end;

procedure TDLHeaderForm.TimeSynchEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Time Synchronization',TimeSynchEntry.Text);
end;

procedure TDLHeaderForm.PositionEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Position',PositionEntry.Text);
end;
procedure TDLHeaderForm.LocationNameEntryChange(Sender: TObject);
begin
     vConfigurations.WriteString(INISection,'Location Name',LocationNameEntry.Text)
end;

initialization
  {$I unit4.lrs}

end.

